import re

file_location = "/home/cwoodall/Documents/Projects/CMS_gadget/firmware/phase2/Phase2/src/vhdl/ipbus/slaves.vhd"

with file(file_location) as fi:
    vhdl_lines = fi.read().split("\n")

class AddressTableEntry(object):
    def __init__(self, name, regAddr, R=True, W=True, mask=0xFFFFFFFF, desc=""):
        self.name = name
        self.regAddr = regAddr
        self.R = R
        self.W = W
        self.mask = mask
        self.desc = desc
        
    def __str__(self):
        if self.desc:
            f_desc = "\n".join(["* "+l for l in self.desc.split('\n')])
        else:
            f_desc = ""

        return "\n%s\n%-24s\t0x%.8x\t0x%.8x\t%d\t%d" % (f_desc, self.name, self.regAddr, self.mask, self.R, self.W)
    
class VHDL2addrTableParser(object):
    def __init__(self, lines):
        self.lines = lines
        self.inTable = False
        self.inEntry = False
        self.results = []

    def parse(self):
        name = ""
        addr = 0x0
        read = True
        write = True
        desc = ""
        
        for line in vhdl_lines:
            if not self.inTable:
                if line.strip() == "--!START_ADDRESS_TABLE":
                    self.inTable = True
            else:
                if line.strip() == "--!END_ADDRESS_TABLE":
                    self.inTable = False
                else:
                    if not self.inEntry:
                        res = re.search("-- ([A-Za-z_0-9]+) \(([RW/]+)\)", line.strip())
                        if res:
 #                           print res.group(1)
 #                          print res.group(2)
                            self.inEntry = True

                            name = res.group(1)
                            if "R" in res.group(2):
                                read = True
                            else:
                                read = False

                            if "W" in res.group(2):
                                write = True
                            else:
                                write = False
                    else:
                        # If starts with a comment line
                        if line.strip()[0:2] == '--':
                            desc += (line.strip()[2:]).strip()
                        else:
                            res = re.search('when x"([0-9A-Fa-f]+)"', line.strip())
                            if res:
#                                print res.group(1)
                                addr = int(res.group(1), 16)
                                self.inEntry = False
#                                print desc
                                self.results.append(AddressTableEntry(name, addr, read, write,desc=desc))
                                desc = ""
                                name = ""
                                read = True
                                write = True
                                addr = 0x0

parser = VHDL2addrTableParser(vhdl_lines)
parser.parse()
print ""

with file("addrTable.txt", "w") as addrTable:
    addrTable.write("*%-23s\t%-10s\t%-10s\tR\tW\n" % ("Register Name", "Address", "Mask"))
    addrTable.write("*----------------------------------------------------------------------------\n")
    for res in parser.results:
        addrTable.write(str(res))
