----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    10:21:11 09/10/2012 
-- Design Name: 
-- Module Name:    ttc_broadcast_commands - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.all;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity ttc_bc0_transmitter is
  port (
    -- Control Interface
    clk40 : in std_logic;
    rst_n : in std_logic;

    -- Turn on orbit signals
    orbit_enable      : in std_logic;
--    initialize_enable : in std_logic;

    bc0_d    : in  std_logic_vector(7 downto 0);
    bc0_wr   : in  std_logic;
    bc0_sent : out std_logic;
    orbit_strobe : out std_logic;
    orbit_count : out unsigned ( 31 downto 0 );
    bc0_out : out std_logic
    );  
end ttc_bc0_transmitter;

architecture Behavioral of ttc_bc0_transmitter is
  -- Interface
--  signal bc0_wr_p : std_logic := '0'; OBSOLETE CODE - Chris W. 2012-120-05
  signal bc0_data_buf : std_logic_vector(12 downto 0);  -- Data + Hamming code
  signal bc0_hamming : std_logic_vector(4 downto 0);

  -- Synchronization
  signal ready : std_logic := '0';
  signal sending : std_logic := '0';
  signal sent : std_logic := '0';
  signal message_counter : integer range 0 to 16 := 0;
  signal send_orbit : std_logic := '0';
  
  -- Orbit controls
  signal orbit_message : std_logic_vector (12 downto 0) := "0000100100010";
  signal orbit_strobe_buf : std_logic;
  signal orbit_lock : std_logic;        -- ORBIT LOCKOUT SIGNAL
  signal orbit_ctr : integer range 0 to 3564 := 0;
begin

  -- Obsolete Code - Chris W. 2012-10-05
  --bc0_wr_prev : process( clk40 )
  --begin
  --  if rising_edge(clk40) then
  --    if rst_n = '0' then
  --      bc0_wr_p <= '0';
  --    else
  --      bc0_wr_p <= bc0_wr;
  --    end if;
  --  end if;
  --end process;

  -- Orbit
  orbit_count <= to_unsigned(orbit_ctr, 32);  -- Acts as an output buffer
  orbit_strobe <= orbit_strobe_buf;           -- Output orbit strobe
  orbit_counter: process(clk40)
  begin
    if rising_edge(clk40) then
      if rst_n = '0' then
        orbit_ctr <= 0;
        orbit_lock <= '0';
        orbit_strobe_buf <= '0';
      else
        -- When orbit_enable is set then BC0 orbit messages will be sent once
        -- an orbit, or every 3563 counts. This will raise the orbit_strobe_buffer,
        -- once an orbit.
        --
        -- The orbit_ctr variable is also available to the outside world through
        -- the unsigned signal orbit_count.
        if orbit_enable = '1' then
          if orbit_ctr = 3563 then
            orbit_strobe_buf <= '1';
            orbit_ctr <= 0;
          else
            orbit_ctr <= orbit_ctr + 1;          
            orbit_strobe_buf <= '0';
          end if;

           -- At this point we cant start sending a command without
           -- stepping on the orbit broadcast, so lock it out.
 
          if orbit_ctr >= 3543 then
            orbit_lock <= '1';
          else
            orbit_lock <= '0';
          end if;
        else
          orbit_lock <= '0';
          orbit_strobe_buf <= '0';
          orbit_ctr <= 0;
        end if;
      end if;
    end if;
  end process;

  send_bc0 : process(clk40)
  begin
    if rising_edge(clk40) then
      if rst_n = '0' then
        orbit_message <= "0000100101010";
        sent <= '0';
        sending <= '0';
        bc0_out <= '1';
        send_orbit <= '0';
      else
        bc0_sent <= sent;
        if sending = '1' then
          if (message_counter = 1) then
            bc0_out <= '0';             -- Send FMT to indicate a broadcast message
            message_counter <= message_counter + 1;
          elsif (message_counter < 15) then
            if (send_orbit = '1') then
              bc0_out <= orbit_message(12);
              orbit_message <= orbit_message(11 downto 0) & orbit_message(12);
            else
              bc0_out <= bc0_data_buf(12);
              bc0_data_buf <= bc0_data_buf(11 downto 0) & '0';
            end if;
            message_counter <= message_counter + 1;
          else
            send_orbit <= '0';
            sending <= '0';
            sent <= '1';
            bc0_out <= '1';
            message_counter <= 0;            
          end if;
        elsif ready = '1' and sent = '0' and orbit_lock = '0' then
          sending <= '1';
          message_counter <= 1;
          bc0_out <= '0';               -- Send Start Bit          
        elsif orbit_strobe_buf = '1' then
          send_orbit <= '1';
          ready <= '1';
        elsif ready = '0' then
          message_counter <= 0;
          bc0_out <= '1';
          if bc0_wr = '1' then
            -- Load message from bc0_d into the data buffer
            bc0_data_buf(12 downto 5) <= bc0_d;            

            -- Calculate Hamming Code for sending messages
            -- hmg[0] = d[0]^d[1]^d[2]^d[3];
            -- hmg[1] = d[0]^d[4]^d[5]^d[6];
            -- hmg[2] = d[1]^d[2]^d[4]^d[5]^d[7];
            -- hmg[3] = d[1]^d[3]^d[4]^d[6]^d[7];
            -- hmg[4] = d[0]^d[2]^d[3]^d[5]^d[6]^d[7];              
            bc0_data_buf(0) <= bc0_d(0) xor bc0_d(1) xor bc0_d(2) xor bc0_d(3);
            bc0_data_buf(1) <= bc0_d(0) xor bc0_d(4) xor bc0_d(5) xor bc0_d(6);
            bc0_data_buf(2) <= bc0_d(1) xor bc0_d(2) xor bc0_d(4) xor bc0_d(5) xor bc0_d(7);
            bc0_data_buf(3) <= bc0_d(1) xor bc0_d(3) xor bc0_d(4) xor bc0_d(6) xor bc0_d(7);
            bc0_data_buf(4) <= bc0_d(0) xor bc0_d(2) xor bc0_d(3) xor bc0_d(5) xor bc0_d(6) xor bc0_d(7);
            ready <= '1';
          end if;
        else
          -- No messages to send.
          message_counter <= 0;
          -- Send idle state
          bc0_out <= '1';
        end if;

        -- Could lead to weird control flow...
        if sent = '1' then
          -- If we sent someting set some bits
          sending <= '0';          
          sent <= '0';
          ready <= '0';
        end if;
      end if;
    end if;
  end process;                      -- send_bc0  

end Behavioral;

